unit kryptool;
{
  Der Quelltext wird bitweise gem dem Passwd verwrfelt (gemischt).
  Die Bits werden sequentiell in Abstnden in den Ausgangstext geschrieben, wobei die Abstnde
  sich aus dem Passwd und einem Zufallswert ergeben. Der Zufallswert steht hinter dem Nutzbit
  und ist immer 4 Bit lang.
  Zwischen den Abstnden wird Rauschen eingefgt.
  Damit man nicht Zeichen fr Zeichen des Passwds erraten kann, wird die Startposition im Ausgangstext
  unter Bercksichtigung aller Zeichen im Passwd bestimmt.
  Damit nicht jede Datei die Nutzbits an der gleichen Stelle hat, wird die Berechnung
  der Startposition mit einer zuflligen Stelle im Passwd begonnen. Die ersten 2 Zeichen im Ausgangstext
  enthalten eine Zufallszahl, die bestimmt, mit welcher Stelle im Passwd begonnen werden muss.

  Ideen:



    - ber das Rauschen einen oder mehrere codierte sinnvolle Klartexte legen.
      Diese Dummy-Texte werden mit einem einfachen Passwort verschlsselt.
      Dadurch erhlt ein Angreifer ggf. eine plausible Decodierung, die aber
      nicht dem eigentlichen codierten Text entspricht. Somit kann er nicht sicher sein,
      ob der decodierte Text wirklich das ist, was ich geheim halten will.

    - Eigenen Editor schreiben, der nur zeichenorientiert arbeitet.



   Erledigt:
    * Passworteingabe bercksichtigt Zeitabstnde zwischen den Zeichen
      und erkennt: kurzer oder langer Abstand. Das gibt ein zustzliches Bit
      im Zeichenvorrat.
    * Zeichen wahlweise per Tastatur oder eigener Bildschirmtastatur eingeben lassen
    * Das Passwd nie im Klartext im Speicher liegen haben. Beim FormCreate
      wird ein zuflliger Schlssel zu dessen Verschlsselung erstellt.

    * Im Speicher nie den decodierten Text halten. Alle Zugriffe ber Mischindex.
      Ausgabe auf Bildschirm zeichenweise mit TextOut (eigene Draw-Routine)
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, ExtDlgs, ExtCtrls, ComCtrls, keyboard
  ;


const
  map:string = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_ ';
  dspFastmode=false;
  dspSlowmotion=true;
type

  TDynamicArray     = array of Integer;
  TDynamicCharArray = array of char;
  TDynamicBoolArray = array of boolean;

  Tverpacker = class(TObject)
  public
    constructor create;
    procedure   clear;
    procedure   addBit(bit:boolean);
    function    getCoded:string;
  private
    dest:       string;
    bitMask:    byte;
    lastByte:   byte;
    linelen:    integer;
  end;

  Tentpacker = class(TObject)
  public
    constructor create;
    procedure   setSource(src:string);
    function    getBit:boolean;
    function    endOfText:boolean;
    function    bitzahl:integer;
  private
    source:     string;
    BitMask:    byte;
    ByteNr:     integer;
    eot:        boolean;
  end;

  Tbitobj = class(TObject)
  public
    constructor create;
    procedure   setSource(src:string);
    function    getBit:boolean;
    function    endOfText:boolean;
    function    bitzahl:longint;
  private
    source:     string;
    BitMask:    word;
    ByteNr:     integer;
    eot:        boolean;
  end;

  Tbyteobj = class(TObject)
  public
    constructor create;
    procedure   clear;
    procedure   addBit(bit:boolean);
    function    getDecoded:string;
  private
    dest:       string;
    bitMask:    word;
    lastByte:   byte;
  end;


  TPassword = class(TObject)
  private
    PasswdMischliste: Array [0..1000] of integer;
    PasswdArray:      Array [0..1000] of char;
    Zeichenzeit1,
    Zeichenzeit2,
    Zeichenzeit3:     TDateTime;
    letztesZeichen:   char;
    zusatzbit:        boolean;
    function    getitem(index:integer):char;
  public
    len:              integer;
    constructor create;
    destructor  destroy;  override;
    procedure   addChar(c:char);
    procedure   clear;
    property    item[Index:integer]:char read GetItem;  default;
  end;





  TFormMain = class(TForm)
    Memo1: TMemo;
    ScrollBar1: TScrollBar;
    Memo2: TMemo;
    ButtonSave: TButton;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    OpenDialog1: TOpenDialog;
    ProgressBar1: TProgressBar;
    Button4: TButton;
    Timer1: TTimer;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ScrollBar1Change(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MenuCodierteDateieditierenClick(Sender: TObject);
    procedure MenuUnverschlsselteTextdateiladenClick(Sender: TObject);
    procedure extverschlsseltspeichern1Click(Sender: TObject);
    procedure secureDisplay;
    procedure Ansehen1Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private-Deklarationen }
    Mischliste:      TDynamicArray;
    gemischterText:  string;
    goingDown:       boolean;
    procedure createDecodeMap;
  public
    en:           Tverpacker;
    de:           Tentpacker;
    bits:         Tbitobj;
    bytes:        Tbyteobj;
    myPasswd:     TPassword;

    decoderMap:   array[0..255] of byte;
    zeilenlaenge: integer;
    function  calcStartbitPos(Passwd: TPassword; offset:integer): integer;
    procedure mischIndexErstellen(txtlen:integer; Passwd:TPassword; var Liste:TDynamicArray);
    function  mischen(txt:string; Liste:TDynamicArray):string;
    function  entmischen(txt:string; Liste:TDynamicArray):string;
    function  getBit(x:byte; bitNr:byte):boolean;

    function  codiere(src:string; Passwd:TPassword; maxlen:integer):string;
    function  decodiere(src:string; Passwd:TPassword):string;
    function  GarbageBit:boolean;

    procedure display1(startline:integer; txt:string; Liste:TDynamicArray; slowmotion:boolean);
    function  ZeilenAufFesteBreite(textliste:Tstrings; breite:integer):string;
    function  leseDatei(filename: string):string;
    function  maximaleZeilenlaenge:integer;
    procedure Tastatur;



    { Public-Deklarationen }
  end;
var
  FormMain:     TFormMain;
  FormKeyboard: TFormKeyboard;

implementation

{$R *.dfm}


function TFormMain.mischen(txt:string; Liste:TDynamicArray):string;
//Zeichen von txt gem Eintrgen in Liste neu anordnen
var
  i,a:     integer;
  txtLen:  integer;
  dest:    string;
  c:       char;
begin
  dest:='';
  txtLen:=length(txt);
  //Mischen
  for i:=0 to txtLen-1 do begin
    a:=Liste[i];
    c:=txt[a];
    dest:=dest+c;
  end;
  result:=dest;
end;

function TFormMain.entmischen(txt: string; Liste:TDynamicArray): string;
//Zeichen von txt gem Eintrgen in Liste neu anordnen
var
  i,a:      integer;
  txtLen:   integer;
  dest:     TDynamicCharArray;
  c:        char;
begin
  txtLen:=length(txt);
  setlength(dest,txtLen+2);
  //entmischen
  for i:=0 to txtLen-1 do begin
    a:=Liste[i];
    c:=txt[i+1];
    dest[a]:=c;
  end;
  for i:=1 to txtLen
  do result:=result+dest[i];
end;


procedure TFormMain.extverschlsseltspeichern1Click(Sender: TObject);
var
  fname, path, txt1, txt2, txt3:string;
  ausgabetext:Tstringlist;
  i, maxlen:integer;
begin
  cursor:=crHourGlass;
  FormMain.Canvas.brush.Color:=$fafffa;
  FormMain.Canvas.Rectangle(0,0,width,height);
  buttonSave.Visible:=false;
  Button4.Visible:=false;

  Application.ProcessMessages;

  maxlen:=maximaleZeilenlaenge;
  txt1:=ZeilenAufFesteBreite(memo2.lines, maxlen);
  mischIndexErstellen(length(txt1),myPasswd,Mischliste);
  txt2:=mischen(txt1,Mischliste);
  txt3:=codiere(txt2,myPasswd,maxlen);

  fname:=ChangeFileExt(extractFilename( Application.ExeName),'.txt');
  path:=ExtractFilePath( Application.ExeName);
      //Text im Speicher lschen
      //So geht's leider nicht:
//      for i:=0 to length(memo2.lines.text) do memo2.lines.text[i]:=#32;

  ausgabetext:=Tstringlist.Create;
  ausgabetext.Text:=txt3;
  ausgabetext.SaveToFile(path+fname);
  ausgabetext.Free;
  memo2.visible:=false;
  Application.ProcessMessages;
  //Speicher versuchen zu lschen, indem einmal etwas kurzes, dann etwas lngeres geschrieben wird
  for i:=0 to memo2.lines.count-1
  do  begin
    memo2.Lines[i]:='123456789 ';
  end;
  for i:=0 to memo2.lines.count-1
  do  begin
    memo2.Lines[i]:='********* ********* ********* ********* ********* ********* ********* ********* ********* ********* ********* ********* ********* ';
  end;
  memo2.Lines.Clear;

  memo1.Lines.Clear;
  memo1.color:=clBlue;
  memo1.Lines.Add(' Datei '+path+fname+' gespeichert.');
  memo1.Lines.Add(' Original knnte noch im Speicher sein - zum lschen den Rechner stromlos machen.');
  memo1.Visible:=true;
  myPasswd.clear;
  cursor:=crDefault;
end;

procedure TFormMain.Ansehen1Click(Sender: TObject);
begin
  memo2.clear;
  memo2.visible:=false;
  Button1.Visible:=false;
  Button2.Visible:=false;
  Button3.Visible:=false;
  cursor:=crHourGlass;
  Application.ProcessMessages;
  secureDisplay;
  cursor:=crDefault;
end;

function TFormMain.calcStartbitPos(Passwd: TPassword; offset:integer): integer;
//offset bestimmt, mit welchem Zeichen des Passwds begonnen wird.
const
  //Irgendwelche Faktoren grer Null
  mult:array[0..10]of integer=(13,3,7,3,1,5,2,1,7,3,1);
var
  i:          integer;
  p:          integer;
  multindex:  integer;
  Passwdindex:integer;
begin
  p:=1;
  multindex:=0;
  for i:= 0 to Passwd.Len-1 do begin
    Passwdindex:=(i+offset) mod Passwd.len;
    p:=p+(ord(Passwd[1+Passwdindex])-31) * mult[multindex];
    inc(multindex);
    if multindex>10 then multindex:=0;
  end;
  result:=p;
end;


function TFormMain.codiere(src:string; Passwd:TPassword; maxlen:integer):string;
var
  einBit:        boolean;
  i,k,n:         integer;
  PasswdIndex:   integer;
  start:         integer;
  offset1:       integer;
  offset2:       integer;
  offset:        integer;
  Zusatzabstand: integer;
begin
  randomize;
  bits.setSource(src);
  en.clear;
  PasswdIndex:=1;

  offset1:=random(64);
  offset2:=random(64);
  offset:=(offset1*64+offset2) mod Passwd.len;

  start:=calcStartbitPos(Passwd, offset)+offset1+offset2;
  for i:=0 to start do en.addBit(GarbageBit);

  ProgressBar1.max:=bits.bitzahl;
  ProgressBar1.position:=0;
  ProgressBar1.Visible:=true;
  n:=0;
  while not bits.endOfText do begin
    Zusatzabstand:=random(16);
    en.addBit((Zusatzabstand and 1) <>0);
    en.addBit((Zusatzabstand and 2) <>0);
    en.addBit((Zusatzabstand and 4) <>0);
    en.addBit((Zusatzabstand and 8) <>0);
    k:=ord(Passwd[PasswdIndex]);
    inc(n);
    inc(PasswdIndex);
    if PasswdIndex > Passwd.len then begin
      PasswdIndex:=1;
      ProgressBar1.position:=n;
    end;
    for i:=0 to k+Zusatzabstand do en.addBit(GarbageBit);
    einBit:=bits.getBit;
    en.addBit(einBit);

  end;
  ProgressBar1.Visible:=false;
  //2 Bytes fr Offsets, die Zeilenlnge, ein Klartext-Spruch und
  //dann der codierte Text
  result:=chr(offset1+48)+chr(offset2+48)+intToStr(maxlen)+'; Vielleicht ist eine Nadel im Heuhaufen, vielleicht ist es Sand'#13#10+ en.getCoded;
end;



function TFormMain.decodiere(src:string; Passwd:TPassword): string;
var
  einBit:          boolean;
  i,n,k:           integer;
  PasswdIndex:     integer;
  start:           integer;
  offset1:         integer;
  offset2:         integer;
  offset:          integer;
  Zusatzabstand:   integer;
  txt:             string;
begin

  PasswdIndex:=1;
  offset1:=ord(src[1])-48;
  offset2:=ord(src[2])-48;
  offset:=(offset1*64+offset2) mod Passwd.len;
  k:=pos(#10, src);
  txt:=copy(src,k+1,length(src));
  de.setSource(txt);

  ProgressBar1.max:=de.bitzahl;
  ProgressBar1.position:=0;
  ProgressBar1.Visible:=true;
  n:=0;

  bytes.clear;
  start:=calcStartbitPos(Passwd, offset)+offset1+offset2;
  for i:=0 to start do de.getBit;

  while not de.endOfText do begin
    k:=ord(Passwd[PasswdIndex]);
    inc(PasswdIndex);
    if PasswdIndex>Passwd.len then begin
      PasswdIndex:=1;
      ProgressBar1.position:=n;
    end;
    Zusatzabstand:=0;
    if de.getbit then Zusatzabstand:=Zusatzabstand or 1;
    if de.getbit then Zusatzabstand:=Zusatzabstand or 2;
    if de.getbit then Zusatzabstand:=Zusatzabstand or 4;
    if de.getbit then Zusatzabstand:=Zusatzabstand or 8;
    inc(n,4);

    //Rauschen berlesen
    for i:=0 to k+Zusatzabstand do begin
      de.getbit;
      inc(n);
    end;
    einBit:=de.getBit;
    inc(n);
    bytes.addBit(einBit);
  end;
  ProgressBar1.Visible:=false;
  result:=bytes.getDecoded;
end;




procedure TFormMain.display1(startline: integer; txt: string;
                             Liste: TDynamicArray; slowmotion:boolean);
const
  lineheight=14;
  charwidth=7;
var
  i,a:            integer;
  txtLen:         integer;
  c:              char;
  x,y:            integer;
  teiler:         integer;
  zeilenJeSeite:  integer;
begin
  if zeilenlaenge < 1 then exit;
  FormMain.width:=charwidth*zeilenlaenge+30;
  ScrollBar1.left:=FormMain.width-24;
  ScrollBar1.LargeChange:=((clientheight-40) div lineheight) -1;
  Application.ProcessMessages;
  FormMain.Canvas.Font.Name:='Courier New';
  FormMain.Canvas.Font.Color:=clBlack;
  FormMain.Canvas.brush.Color:=$fffafa;
  FormMain.Canvas.Rectangle(0,0,width,height);
  txtlen:=length(txt);
  ScrollBar1.Max:=txtlen div zeilenlaenge;
  ScrollBar1.top:=0;
  ScrollBar1.height:=clientheight;
  teiler:=10;
  zeilenJeSeite:=clientheight div lineheight;
  //Entmischen und dann anzeigen
  for i:=0 to txtLen-1 do begin
    if goingDown then exit; //Wenn Anwender Programm schliet: Schleife verlassen

    if slowmotion and (ScrollBar1.position < 1) and  (0=(i mod teiler)) then begin
      sleep(8);
      teiler:=teiler+40;
      Application.ProcessMessages;
    end;
    if 0=(i mod 512) then Application.ProcessMessages;
    if goingDown then exit;
    a:=Liste[i]-1;
    y:=a div zeilenlaenge;
    x:=a mod zeilenlaenge;
    c:=txt[i+1];
    if ((y >= startline) and (y <= startline + zeilenJeSeite)) and (ord(c)>32)
    then FormMain.Canvas.TextOut(2+x*charwidth,2+(y-startline)*lineheight, c);
  end;
end;

procedure TFormMain.createDecodeMap;
//Array fr mein uudecode erstellen
var
  i,p:integer;
begin
  for i:=0 to 255 do begin
    p:=pos(chr(i),map);
    if p>0
    then decoderMap[i]:=p-1
    else decoderMap[i]:=0;
  end;
end;


procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i:integer;
begin
  goingDown:=true;
  Application.ProcessMessages;
  for i:=0 to high(Mischliste) do Mischliste[i]:=0;

  de.Free;
  en.free;
  bits.Free;
  bytes.Free;
  mypasswd.free; //<-gibt ggf. Probleme, wenn whrend des Display1 auf Close geklickt wird!

end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  randomize;
  createDecodeMap;
  en:=Tverpacker.Create;
  de:=Tentpacker.Create;
  bits:=Tbitobj.Create;
  bytes:=Tbyteobj.Create;
  myPasswd:=TPassword.create;
  en.clear;
  bytes.clear;
  memo1.visible:=false;
  memo2.visible:=true;
  goingDown:=false;
end;

function TFormMain.GarbageBit: boolean;
begin
  result:=random(2) > 0;
  //zu Testzwecken kann eine der nachfolgenden Zeile aktiviert werden. Dann sieht man nur
  //die gesetzten/gelschten Nutzbits und kein Rauschen.
//  result:=false;
//  result:=true;
end;

function TFormMain.getBit(x, bitNr: byte): boolean; //BitNr: 0..7
begin
  result:=(x and (1 shl bitNr))<>0;
end;

function TFormMain.leseDatei(filename: string):string;
var
  quelltext: Tstringlist;
  p:         integer;
begin
  quelltext:=Tstringlist.Create;
  quelltext.LoadFromFile(filename);
  p:=pos(';',quelltext[0]);
  if p>0
  then zeilenlaenge:=strToIntDef(copy(quelltext[0], 3, p-3),120)
  else zeilenlaenge:=120;
  result:=quelltext.text;
end;


function TFormMain.maximaleZeilenlaenge: integer;
//bestimmt Lnge der lngsten Zeile im Memo2
var
  i, len: integer;
  maxlen: integer;
begin
  maxlen:=0;
  for i := 0 to memo2.lines.Count - 1
  do begin
    len:=length(memo2.lines[i]);  //ggf. trimRight benutzen
    if len > maxlen then maxlen:=len;
  end;
  result:=maxlen+1;
end;
{******************************************************************************}



procedure TFormMain.MenuCodierteDateieditierenClick(Sender: TObject);
var
  txt:          string;
  zeile:        string;
  fname:        string;
  textEnde:     integer;
  i,n,p:        integer;
  zeichen:      TDynamicCharArray;
  c:            char;
begin
  fname:=ChangeFileExt(extractFilename( Application.ExeName),'.txt');
  if fileexists(fname) then begin
    Button1.Visible:=false;
    Button2.Visible:=false;
    Button3.Visible:=false;
    memo2.clear;

    FormMain.Canvas.brush.Color:=$4040ff;
    FormMain.Canvas.Rectangle(0,0,width,height);
    Application.ProcessMessages;

    if myPasswd.len > 1 then begin
      //Datei lesen und dekodieren
        cursor:=crHourGlass;
        Application.ProcessMessages;

        txt:=leseDatei(fname);
        gemischterText:=decodiere(txt, myPasswd); //entmischen erfolgt bei der Bildschirmausgabe
        textEnde:=length(gemischterText)-1;
        mischIndexErstellen(textEnde, myPasswd, Mischliste);
        setlength(zeichen,textEnde+1);
        for i:= 0 to textEnde do begin
          p:=Mischliste[i];
          c:=gemischterText[i+1];
          zeichen[p]:=c;
        end;
        zeile:='';
        memo2.Clear;
        n:=0;
        for i:= 0 to textEnde do begin
          if ord(zeichen[i]) > 31
          then zeile:=zeile+zeichen[i];
          inc(n);
          if n>=zeilenlaenge then begin
            n:=0;
            memo2.Lines.add(zeile);
            zeile:='';
          end;

        end;
        Canvas.Rectangle(0,0,width,height);
        cursor:=crDefault;
        memo2.Visible:=true;
        Button4.Visible:=true;
        ButtonSave.Visible:=true;
    end;
  end;
end;

procedure TFormMain.MenuUnverschlsselteTextdateiladenClick(Sender: TObject);
begin
  OpenDialog1.Filter := 'Textdateien (*.txt)|*.TXT|Alle Dateien |*';
  if OpenDialog1.Execute then begin
    Memo2.Lines.LoadFromFile(OpenDialog1.FileName);
    ButtonSave.Visible:=true;
    Button1.Visible:=false;
    Button2.Visible:=false;
    Button4.Visible:=true;
    Button3.Visible:=false;
    memo2.Visible:=true;
  end;

end;

procedure TFormMain.mischIndexErstellen(txtlen:integer; Passwd:TPassword; var Liste:TDynamicArray);
var
  i,a,c,x,p:   integer;
  PasswdIndex: integer;
begin

  //Liste initialisieren
  SetLength(Liste, txtLen+1);
  for i:=0 to txtLen-1 do begin
    Liste[i]:=i+1;
  end;

  p:=0;
  PasswdIndex:=1;
  //Mischen
  //in kleinen Schritten rauf
  for i:=0 to txtLen-1 do begin
    c:=ord(Passwd[PasswdIndex]);
    inc(PasswdIndex);
    if PasswdIndex>Passwd.len then PasswdIndex:=1;
    p:=p+c;
    while p >= txtLen do dec(p,txtLen);
    x:=p;
    //vertauschen
    a:=Liste[i];
    Liste[i]:=Liste[x];
    Liste[x]:=a;
  end;
  //in grossen Schritten runter
  p:=txtLen-1;
  for i:=txtLen-1 downto 0 do begin
    c:=7*ord(Passwd[PasswdIndex]);
    inc(PasswdIndex);
    if PasswdIndex>Passwd.len then PasswdIndex:=1;
    p:=p-c;
    while p < 0 do inc(p,txtLen);
    x:=p;
    //vertauschen
    a:=Liste[i];
    Liste[i]:=Liste[x];
    Liste[x]:=a;
  end;
end;

procedure TFormMain.ScrollBar1Change(Sender: TObject);
begin
  display1(ScrollBar1.Position, gemischterText, Mischliste, dspSlowmotion);
end;

procedure TFormMain.secureDisplay;
var
  fname:string;
  txt:string;
  txtlen:integer;
begin
  fname:=ChangeFileExt(extractFilename( Application.ExeName),'.txt');
  if fileexists(fname) then begin

    FormMain.Canvas.brush.Color:=$fffafa;
    FormMain.Canvas.Rectangle(0,0,width,height);
    Application.ProcessMessages;

    if myPasswd.len > 1 then begin
      //Datei lesen und dekodieren
      try
        txt:=leseDatei(fname);
        gemischterText:=decodiere(txt, myPasswd); //entmischen erfolgt bei der Bildschirmausgabe
        txtlen:=length(gemischterText)-1;
        mischIndexErstellen(txtlen, myPasswd, Mischliste);

        ScrollBar1.Max:=length(gemischterText) div zeilenlaenge;
        ScrollBar1.position:=0;
        display1(ScrollBar1.position, gemischterText, Mischliste, dspSlowmotion);
        ScrollBar1.visible:=true;
      except
        memo1.Lines.Clear;
        memo1.color:=clRed;
        memo1.Lines.Add(' Datei '+fname+' nicht decodierbar.');
        memo1.Visible:=true;
        myPasswd.clear;
        exit;
      end;
    end;
  end else begin
    memo1.Lines.Clear;
    memo1.color:=clRed;
    memo1.Lines.Add(' Datei '+fname+' nicht gefunden.');
    memo1.Visible:=true;
  end;
  if not goingDown then myPasswd.clear;

end;

function TFormMain.ZeilenAufFesteBreite(textliste: Tstrings;
  breite: integer): string;
//Alle Zeilen mit Steuerzeichen auf Breite auffllen
var
  i,x,len: integer;
  leer:    string;

begin
  leer:='';
  for i:=1 to breite do leer:=leer+'*';

  for i := 0 to textliste.Count - 1 do begin
    len:=length(textliste[i]);
    for x:=1 to breite-len do leer[x]:=chr(random(24)+9);
    result:=result+copy(textliste[i]+leer,1, breite);
  end;
end;

procedure Tverpacker.addBit(bit: boolean);
begin
  if bit
  then lastByte:=lastByte or bitMask;
  bitMask:=bitMask *2;
  if bitMask>32 then begin
    dest:=dest+map[lastByte+1];
    inc(linelen);
    if linelen > 80 then begin
      linelen:=0;
      dest:=dest+#13#10;
    end;
    lastByte:=0;
    bitMask:=1;
  end;
end;

procedure Tverpacker.clear;
begin
  Bitmask:=1;
  dest:='';
  lastByte:=0;
  linelen:=0;
end;

constructor Tverpacker.create;
begin
  inherited;
  clear;
end;

function Tverpacker.getCoded: string;
begin
  if bitMask=1
  then result:=dest
  else result:=dest+map[lastByte+1];
end;

{ Tentpacker }

function Tentpacker.bitzahl: integer;
begin
  result:=length(source)*8;
end;

constructor Tentpacker.create;
begin
  source:='';
  BitMask:=1;
  ByteNr:=1;
  eot:=true;
end;

function Tentpacker.endOfText: boolean;
begin
  result:=eot;
end;

function Tentpacker.getBit: boolean;
var
  b,m:    byte;
  srcLen: integer;
begin
  srcLen:=length(source);
  if byteNr <= srcLen then begin
    b:=ord(source[ByteNr]);
    while b < 32 do begin //Steuerzeichen berspringen
      inc(ByteNr);
      if ByteNr <=srcLen
      then b:=ord(source[ByteNr])
      else begin
        result:=false;
        eot:=true;
        exit;
      end;
    end;

    m:=FormMain.decoderMap[b];
    result:=(m and BitMask)<>0;
    BitMask:= BitMask*2;
    if BitMask>32 then begin
      BitMask:=1;
      inc(byteNr);
    end;
  end else begin
    result:=false;
    eot:=true;
  end;
end;

procedure Tentpacker.setSource(src: string);
begin
  source:=src;
  BitMask:=1;
  ByteNr:=1;
  eot:=false;
end;

{ Tbitobj }

function Tbitobj.bitzahl: longint;
begin
  result:=length(source)*8;
end;

constructor Tbitobj.create;
begin
  source:='';
  BitMask:=1;
  ByteNr:=1;
  eot:=true;
end;

function Tbitobj.endOfText: boolean;
begin
  result:=eot;
end;

function Tbitobj.getBit: boolean;
var
  b:byte;
begin
  if byteNr <= length(source) then begin
    b:=ord(source[ByteNr]);
    result:=(b and BitMask)<>0;
    BitMask:= BitMask*2;
    if BitMask>255 then begin
      BitMask:=1;
      inc(byteNr);
    end;
  end else begin
    result:=false;
    eot:=true;
  end;
end;

procedure Tbitobj.setSource(src: string);
begin
  source:=src;
  BitMask:=1;
  ByteNr:=1;
  eot:=length(src)=0;
end;

{ Tbyteobj }

procedure Tbyteobj.addBit(bit: boolean);
begin
  if bit
  then lastByte:=lastByte or bitMask;
  bitMask:=bitMask *2;
  if bitMask>255 then begin
    dest:=dest+chr(lastByte);
    lastByte:=0;
    bitMask:=1;
  end;
end;

procedure Tbyteobj.clear;
begin
  dest:='';
  bitMask:=1;
  lastByte:=0;
end;

constructor Tbyteobj.create;
begin
  inherited;
  clear;
end;

function Tbyteobj.getDecoded: string;
begin
  if bitMask=1
  then result:=dest
  else result:=dest+chr(lastByte);
end;

{ TPasswd }

procedure TPassword.addChar(c: char);
begin
  Zeichenzeit3:=Zeichenzeit2;
  Zeichenzeit2:=Zeichenzeit1;
  Zeichenzeit1:=now;
  if   letztesZeichen = ' ' then begin
    if ((Zeichenzeit1 - Zeichenzeit2) < (Zeichenzeit2 - Zeichenzeit3)) and (len>1)
    then zusatzbit:=true
    else zusatzbit:=false;
  end;
  letztesZeichen:=c;
  if zusatzbit
  then c:=chr(ord(c) xor 128);
  inc(len);
  PasswdArray[PasswdMischliste[len]]:=c;
  
end;

function TPassword.getitem(index: integer): char;
begin
  result:=PasswdArray[PasswdMischliste[Index]];
end;


procedure TPassword.clear;
var
  i:integer;
  x,tmp:integer;
begin
  len:=0;
  //Passwd mit Rauschen fllen
  for i:=low(PasswdArray) to high(PasswdArray)
  do PasswdArray[i]:=chr(random(256));

  //Init Mischliste
  for i:=low(PasswdMischliste) to high(PasswdMischliste)
  do Passwdmischliste[i]:=i;
  //Mischen
  for i:=low(PasswdMischliste) to high(PasswdMischliste)
  do begin
     x:=random(high(PasswdMischliste)+1);
     tmp:=PasswdMischliste[i];
     PasswdMischliste[i]:=PasswdMischliste[x];
     PasswdMischliste[x]:=tmp;
  end;
  letztesZeichen:=#0;
  zusatzbit:=false;
  Zeichenzeit2:=now;
  Zeichenzeit3:=now;
end;

constructor TPassword.create;
begin
  inherited;
  clear;
end;


destructor TPassword.destroy;
begin
  clear;
  inherited;
end;

procedure TFormMain.Button4Click(Sender: TObject);
begin
  OpenDialog1.DefaultExt:='.txt';
  OpenDialog1.Filter := 'Textdateien (*.txt)|*.TXT|Alle Dateien |*';
  if OpenDialog1.execute
  then memo2.Lines.SaveToFile(OpenDialog1.FileName);
end;


procedure TFormMain.Tastatur;
var
  kbd:TFormKeyboard;
begin
  kbd:=TFormKeyboard.Create(Application);
  (* Hier kann man seine eigene Tastatur zusammenbauen lassen
     Jedes Zeichen wird als Taste dargestellt. Folgende Spezialzeichen gibt es: 
     '|00'   Zeilentrenner, gefolgt von 2 Zeichen Einrcktiefe in 1/10 Tastenbreite
     '& '    Leertaste (4 Tasten breit)
     '&&'    Das & Zeichen
     '&A'..'&Z'    Tasten-Farbschema
     '&_'    Leerraum
     '&1'.. '&9'    1/10..9/10 Leerraum

  *)

  kbd.buildLayout('!"$%&/()=?|001234567890|05QWERTZUIOP*|08ASDFGHJKL''|11YXCVBNM;:_|16&_&_&_& '
                  +'|05&Bqwertzuiop+|08asdfghjkl#|11yxcvbnm,.-|16&_&_&_& '
                  +'|35&C' //die Spezialzeichen mit Alt-3Ziffern-auf-NumKeypad eingeben
                  );

  kbd.BorderIcons:=[];
  kbd.BorderStyle:=bsNone;
  kbd.top:=0;
  kbd.left:=0;
  kbd.title:='Zugangscode:';
  kbd.caption:='Eingabe des Zugangscodes - Am Ende Eingabetaste drcken';
//  kbd.parent:=self; <-Darf nicht, sonst kommen keine Ereignisse an.
  kbd.BorderIcons:=[];
  kbd.Position:=poScreenCenter;
  kbd.BorderStyle:=bsSingle;
  kbd.top:=50+random(100);
  kbd.left:=50+random(100);

  kbd.showmodal;
  kbd.free;

  FormMain.Canvas.brush.Color:=$ffffff;
  FormMain.Canvas.Rectangle(0,0,width,height);

  Button1.Visible:=true;
  Button2.Visible:=true;
  Button3.Visible:=true;
  Label1.Visible:=false;
end;

procedure TFormMain.FormResize(Sender: TObject);
begin
   display1(ScrollBar1.Position, gemischterText, Mischliste, dspFastmode);
end;

procedure TFormMain.Timer1Timer(Sender: TObject);
begin
  Timer1.enabled:=false;
  Tastatur;
end;

end.


